__author__ = 'karl.gong'


class TestListener(object):
    def on_test_suite_start(self, test_suite):
        pass

    def on_test_suite_finish(self, test_suite):
        pass

    def on_test_class_start(self, test_class):
        pass

    def on_test_class_finish(self, test_class):
        pass

    def on_test_group_start(self, test_group):
        pass

    def on_test_group_finish(self, test_group):
        pass

    def on_test_case_start(self, test_case):
        pass

    def on_test_case_finish(self, test_case):
        pass


class TestListenerGroup(object):
    def __init__(self):
        self.__test_listeners = []

    def append(self, test_listener):
        self.__test_listeners.append(test_listener)

    def on_test_suite_start(self, test_suite):
        for test_listener in self.__test_listeners:
            test_listener.on_test_suite_start(test_suite)

    def on_test_suite_finish(self, test_suite):
        for test_listener in self.__test_listeners:
            test_listener.on_test_suite_finish(test_suite)

    def on_test_class_start(self, test_class):
        for test_listener in self.__test_listeners:
            test_listener.on_test_class_start(test_class)

    def on_test_class_finish(self, test_class):
        for test_listener in self.__test_listeners:
            test_listener.on_test_class_finish(test_class)

    def on_test_group_start(self, test_group):
        for test_listener in self.__test_listeners:
            test_listener.on_test_group_start(test_group)

    def on_test_group_finish(self, test_group):
        for test_listener in self.__test_listeners:
            test_listener.on_test_group_finish(test_group)

    def on_test_case_start(self, test_case):
        for test_listener in self.__test_listeners:
            test_listener.on_test_case_start(test_case)

    def on_test_case_finish(self, test_case):
        for test_listener in self.__test_listeners:
            test_listener.on_test_case_finish(test_case)


test_listeners = TestListenerGroup()



__author__ = 'KarlGong'


class FilterGroup:
    def __init__(self):
        self.__filters = []

    def filter(self, attr_ref):
        if self.__filters is None or len(self.__filters) == 0:
            return True
        for ft in self.__filters:
            if not ft.filter(attr_ref):
                return False
        return True

    def append_filter(self, filter):
        self.__filters.append(filter)

    def __str__(self):
        filter_strs = []
        for ft in self.__filters:
            filter_strs.append(str(ft))
        return "\n ".join(filter_strs)


class TestClassNameFilter:
    def __init__(self, name):
        self._name = name

    def filter(self, test_class_ref):
        return self._name is None or self._name == test_class_ref.__name__


class TestCaseNameFilter:
    def __init__(self, name):
        self._name = name

    def filter(self, test_case_ref):
        return self._name is None or self._name == test_case_ref.__name__


class TestCaseIncludeTagsFilter:
    def __init__(self, tags):
        self._tags = tags

    def filter(self, test_case_ref):
        return self._tags is None or len([val for val in self._tags if val in test_case_ref.__tags__]) != 0

    def __str__(self):
        return "Include Tags: %s" % self._tags


class TestCaseExcludeTagsFilter:
    def __init__(self, tags):
        self._tags = tags

    def filter(self, test_case_ref):
        return self._tags is None or len([val for val in self._tags if val in test_case_ref.__tags__]) == 0

    def __str__(self):
        return "Exclude Tags: %s" % self._tags


class TestCaseIncludeGroupsFilter:
    def __init__(self, groups):
        self._groups = groups

    def filter(self, test_case_ref):
        return self._groups is None or test_case_ref.__group__ in self._groups

    def __str__(self):
        return "Include Groups: %s" % self._groups



import errno
import os

__author__ = 'karl.gong'


def make_dirs(dir_path):
    try:
        os.makedirs(dir_path)
    except OSError as e:
        if e.errno != errno.EEXIST:  # be happy if someone already created the path
            raise


def remove_tree(dir_path, remove_root=True):
    if os.path.exists(dir_path):
        for root, dirs, files in os.walk(dir_path, topdown=False):
            for name in files:
                os.remove(os.path.join(root, name))
            for name in dirs:
                os.rmdir(os.path.join(root, name))
        if remove_root is True:
            os.rmdir(dir_path)


import copy
import importlib
import pkgutil

from .enumeration import PDecoratorType
from .testfilter import TestClassNameFilter, TestCaseNameFilter

__author__ = 'karl.gong'

class TestFinder:
    def __init__(self, test_target, test_class_filter_group, test_case_filter_group, target_test_suite):
        self.test_target = test_target
        self.test_class_filter_group = copy.deepcopy(test_class_filter_group)
        self.test_case_filter_group = copy.deepcopy(test_case_filter_group)
        self.target_test_suite = target_test_suite
        self.found_test_case_count = 0
        self.repeated_test_case_count = 0

    def find_test_case(self):
        splitted_test_target = self.test_target.split(".")
        module_ref = None
        module_name_len = 0
        for i in range(len(splitted_test_target)):
            try:
                module_ref = importlib.import_module(".".join(splitted_test_target[:i + 1]))
                module_name_len = i + 1
            except ImportError as e:
                if splitted_test_target[i] in str(e):
                    break
                raise

        if module_ref is None:
            raise ImportError("Test target <%s> is invalid.\nNo module named <%s>."% (self.test_target, splitted_test_target[0]))

        test_target_len = len(splitted_test_target)
        if module_name_len == test_target_len:
            if hasattr(module_ref, "__path__"):
                # test target is package
                self.find_test_cases_in_package(module_ref)
            else:
                # test target is module
                self.find_test_cases_in_module(module_ref)
        elif module_name_len == test_target_len - 1:
            # test target is class
            self.test_class_filter_group.append_filter(TestClassNameFilter(splitted_test_target[-1]))
            self.find_test_cases_in_module(module_ref)
        elif module_name_len == test_target_len - 2:
            # test target is method
            self.test_class_filter_group.append_filter(TestClassNameFilter(splitted_test_target[-2]))
            self.test_case_filter_group.append_filter(TestCaseNameFilter(splitted_test_target[-1]))
            self.find_test_cases_in_module(module_ref)
        else:
            raise ImportError("Test target <%s> is probably invalid.\nModule <%s> exists but module <%s> doesn't."% (
                self.test_target, ".".join(splitted_test_target[:module_name_len]), ".".join(splitted_test_target[:module_name_len + 1])))

    def find_test_cases_in_package(self, package_ref):
        for _, name, is_package in pkgutil.iter_modules(package_ref.__path__):
            if is_package:
                self.find_test_cases_in_package(importlib.import_module(package_ref.__name__ + "." + name))
            else:
                self.find_test_cases_in_module(importlib.import_module(package_ref.__name__ + "." + name))

    def find_test_cases_in_module(self, module_ref):
        for module_element in dir(module_ref):
            test_class_ref = getattr(module_ref, module_element)
            try:
                if test_class_ref.__pd_type__ == PDecoratorType.TestClass \
                        and test_class_ref.__enabled__ \
                        and test_class_ref.__module__ == module_ref.__name__ \
                        and self.test_class_filter_group.filter(test_class_ref):
                    self.find_test_cases_in_class(test_class_ref)
            except AttributeError:
                pass

    def find_test_cases_in_class(self, test_class_ref):
        for class_element in dir(test_class_ref):
            test_case_ref = getattr(test_class_ref(), class_element)
            try:
                if test_case_ref.__pd_type__ == PDecoratorType.Test \
                        and test_case_ref.__enabled__ \
                        and self.test_case_filter_group.filter(test_case_ref):
                    self.found_test_case_count += 1
                    if not self.target_test_suite.add_test_case(test_case_ref):
                        self.repeated_test_case_count += 1
            except AttributeError:
                pass


import logging
import sys

from . import config

__author__ = 'karl.gong'


class PConsole:
    def __init__(self, out):
        self.out = out

    def write(self, msg):
        self.out.write(msg)

    def write_line(self, msg):
        self.write(msg + "\n")


pconsole = PConsole(sys.stdout)


class PReporter:
    def __init__(self):
        pass

    def debug(self, msg):
        self.__log(logging.DEBUG, msg)

    def info(self, msg):
        self.__log(logging.INFO, msg)

    def warn(self, msg):
        self.__log(logging.WARN, msg)

    def error(self, msg):
        self.__log(logging.ERROR, msg)

    def critical(self, msg):
        self.__log(logging.CRITICAL, msg)

    def __log(self, level, msg):
        from . import testexecutor

        try:
            running_test_fixture = testexecutor.current_executor().get_property("running_test_fixture")
        except AttributeError:
            sys.stdout.write("[%s] %s\n" % (logging.getLevelName(level), msg))
        else:
            running_test_fixture.logs.append({"level": logging.getLevelName(level).lower(), "message": str(msg)})

            if config.get_option("verbose"):
                # output to pconsole
                message = "[%s] %s" % (running_test_fixture.full_name, msg)
                pconsole.write_line(message)


preporter = PReporter()
